//X-box controller

/*
 Controls inputs from xbox controller and send it to client.js
 Made by Joztein Munz, edited by Torkil Eide Solstad
 */


var joystickValue = {RIGHT_STICK_X: 2, RIGHT_STICK_Y: 2, LEFT_STICK_Y: 2, LEFT_STICK_X: 2};
var epsilon = 0.2;


initXBOXControllerHandler();

function initXBOXControllerHandler() {
    var gamepad = new Gamepad();

    gamepad.bind(Gamepad.Event.CONNECTED, function (device) {
        // a new gamepad connected
        console.log("connected new gamepad!");
    });

    gamepad.bind(Gamepad.Event.DISCONNECTED, function (device) {
        // gamepad disconnected
        console.log("gamepad disconnected");
    });

    gamepad.bind(Gamepad.Event.UNSUPPORTED, function (device) {
        // an unsupported gamepad connected (add new mapping)
    });

    gamepad.bind(Gamepad.Event.BUTTON_DOWN, function (e) {
        // e.control of gamepad e.gamepad pressed down
        //console.log("button down. control: " + e.control);
        buttionDown(e.control);
    });

    gamepad.bind(Gamepad.Event.BUTTON_UP, function (e) {
        // e.control of gamepad e.gamepad released
        //console.log("button up. control : " + e.control);
    });

    gamepad.bind(Gamepad.Event.AXIS_CHANGED, function (e) {
        // e.axis changed to value e.value for gamepad e.gamepad
        // console.log("axis changed. axis: " + e.axis + ". value : " + e.value);
        checkIfInitialized(e);
        if (path_mode || (e.value > epsilon || e.value < -epsilon)) { // Avoid sending thrust commands when not using.
            processStickInput(e);
        } else {
            var command = {axis: e.axis, value: 0};
            processStickInput(command);
            //checkPrevious(e, epsilon); //cheak if last was more than epsilon, if it was send command 0.
        }
        updateJoystickValues(e);
    });

    gamepad.bind(Gamepad.Event.TICK, function (gamepads) {
        // gamepads were updated (around 60 times a second)
    });

    if (!gamepad.init()) {
        // Your browser does not support gamepads, get the latest Google Chrome or Firefox
        alert("Controller unsupported. Get latest Google Chrome of Firefox if you want to use gamepad input");

    }
}

function buttionDown(e) {

    //console.log(e);

    switch (e) {
        case "LEFT_TOP_SHOULDER":
            hide_show_topbar();
            break;
        case "RIGHT_TOP_SHOULDER":
            hide_show_info();
            break;
        case "FACE_1":
            //Dummievalues();
            break;
        case "FACE_2":
            /*if (path_mode) {
                updatePathfollowing();
            }*/
            break;
        default:
            return;
    }
}

function processStickInput(e) {
    var dir;
    switch (e.axis) {
        case "RIGHT_STICK_X":
            dir = "sway";
            sendJoystickCommandMotion(e.value, dir);
            break;
        case "RIGHT_STICK_Y":
            dir = "heave";
            sendJoystickCommandMotion(-e.value, dir); //
            break;
        case "LEFT_STICK_Y":
            dir = "surge";
            sendJoystickCommandMotion(-e.value, dir); //
            break;
        case "LEFT_STICK_X":
            dir = "yaw";
            sendJoystickCommandMotion(e.value, dir); //
            break;
        default:
            return;
    }
}

function sendJoystickCommandMotion(value, dir) {
    if (inputMode == "joy") {
        moveJoystickOnScrean(value, dir);
    }
}

function updateJoystickValues(e) {
    switch (e.axis) {
        case "RIGHT_STICK_X":
            joystickValue.RIGHT_STICK_X = e.value;
            break;
        case "RIGHT_STICK_Y":
            joystickValue.RIGHT_STICK_Y = e.value;
            break;
        case "LEFT_STICK_Y":
            joystickValue.LEFT_STICK_Y = e.value;
            break;
        case "LEFT_STICK_X":
            joystickValue.LEFT_STICK_X = e.value;
            break;
        default:
            return;
    }
}

function checkPrevious(e, epsilon) {
    var previous;
    switch (e.axis) {
        case "RIGHT_STICK_X":
            previous = joystickValue.RIGHT_STICK_X;
            break;
        case "RIGHT_STICK_Y":
            previous = joystickValue.RIGHT_STICK_Y;
            break;
        case "LEFT_STICK_Y":
            previous = joystickValue.LEFT_STICK_Y;
            break;
        case "LEFT_STICK_X":
            previous = joystickValue.LEFT_STICK_X;
            break;
        default:
            return;
    }
    if (previous > epsilon || previous < -epsilon) {
        var command = {axis: e.axis, value: 0};
        processStickInput(command);
    }
}

function checkIfInitialized(e) {
    switch (e.axis) {
        case "RIGHT_STICK_X":
            if (joystickValue.RIGHT_STICK_X == 2) {
                joystickValue.RIGHT_STICK_X = e.value;
            }
            break;
        case "RIGHT_STICK_Y":
            if (joystickValue.RIGHT_STICK_Y == 2) {
                joystickValue.RIGHT_STICK_Y = e.value;
            }
            break;
        case "LEFT_STICK_Y":
            if (joystickValue.LEFT_STICK_Y == 2) {
                joystickValue.LEFT_STICK_Y = e.value;
            }
            break;
        case "LEFT_STICK_Y":
            if (joystickValue.LEFT_STICK_X == 2) {
                joystickValue.LEFT_STICK_X = e.value;
            }
            break;
        default:
            return;
    }
}
